﻿
Module Module_Utils

    Public Sub Swap(Of T)(ByRef a As T, ByRef b As T)
        Dim temp As T
        temp = a
        a = b
        b = temp
    End Sub

    ' ========================================================
    '   PROCESS START AND WAIT
    ' ========================================================
    Friend Sub ProcessStartAndWait(ByVal fname As String)
        If Not My.Computer.FileSystem.FileExists(fname) Then Return
        Dim p As Process = Process.Start(fname)
        Do
            System.Threading.Thread.Sleep(100)
        Loop Until p.HasExited
    End Sub

    ' ==============================================================
    '  REPLACE MULTIPLE SPACES and TABS and TRIM  
    '  (3..5 uS for Strings with 32 chars)
    ' ============================================================== 
    Friend Function ReplaceMultipleSpacesAndTrim(ByVal s As String) As String
        s = s.Replace(vbTab, " ")
        While s.Contains("  ")
            s = s.Replace("  ", " ")
        End While
        Return s.Trim
    End Function

    ' ==========================================================================
    '  Convert SSID 802.11 to string
    ' ==========================================================================
    Public Function GetStringForSSID(ByVal ssid As Wlan.Dot11Ssid) As String
        Dim s As String = ""
        For k As Integer = 0 To CInt(ssid.SSIDLength - 1)
            s += Chr(ssid.SSID(k))
        Next
        Return s
    End Function

    ' ==========================================================================
    '  Convert ByteArray to string
    ' ==========================================================================
    Public Function ByteArrayToString(ByVal b() As Byte) As String
        Dim s As String = ""
        For k As Integer = 0 To b.Length - 1
            If b(k) = 0 Then Exit For
            s += Chr(b(k))
        Next
        Return s
    End Function


    ' ==========================================================================
    '  Convert IES to string
    ' ==========================================================================
    Public Function IesToString(ByVal b() As Byte) As String
        Dim s As String = ""
        For k As Integer = 0 To b.Length - 1
            Dim b1 As Byte = b(k)
            If b1 < 32 Or b1 > 127 Then b1 = 32
            s += Chr(b1)
        Next
        Return s
    End Function



    ' ==========================================================================
    '  Convert MacAddress to string
    ' ==========================================================================
    Public Function ByteArrayToMacAddress(ByVal b() As Byte) As String
        Dim s As String = ""
        For k As Integer = 0 To b.Length - 1
            s += b(k).ToString("x2").ToUpper + "-"
        Next
        Return s.TrimEnd("-"c)
    End Function

    ' ==========================================================================
    '  Convert DOT11_PHY_TYPE to string
    ' ==========================================================================
    Public Function PHY_ToString(ByVal phy As Wlan.Dot11PhyType) As String
        If phy < Wlan.Dot11PhyType.IhvStart Then
            Select Case phy
                Case Wlan.Dot11PhyType.Any, Wlan.Dot11PhyType.Unknown : Return "Unknown"
                Case Wlan.Dot11PhyType.Fhss : Return "FHSS"
                Case Wlan.Dot11PhyType.Dsss : Return "DSSS"
                Case Wlan.Dot11PhyType.IrBaseband : Return "Infrared"
                Case Wlan.Dot11PhyType.Ofdm : Return "802.11a"
                Case Wlan.Dot11PhyType.Hrdsss : Return "HRDSS"
                Case Wlan.Dot11PhyType.Erp : Return "802.11g"
                Case Wlan.Dot11PhyType.Ht : Return "802.11n"
                Case Wlan.Dot11PhyType.Vht : Return "802.11ac"
                Case Else : Return "Unknown"
            End Select
        Else
            Return "IHV: " + phy.ToString
        End If
    End Function

    ' ==========================================================================
    '  Convert CenterFrequency to Channel
    ' ==========================================================================
    Public Function CenterFrequencyToChannel(ByVal f As UInt32) As Int32
        Select Case f
            Case 2412000 : Return 1
            Case 2417000 : Return 2
            Case 2422000 : Return 3
            Case 2427000 : Return 4
            Case 2432000 : Return 5
            Case 2437000 : Return 6
            Case 2442000 : Return 7
            Case 2447000 : Return 8
            Case 2452000 : Return 9
            Case 2457000 : Return 10
            Case 2462000 : Return 11
            Case 2467000 : Return 12
            Case 2472000 : Return 13
            Case 2484000 : Return 14
            Case Else : Return 0
        End Select
    End Function

    ' ==========================================================================
    '  Calculate distance
    ' ==========================================================================
    Public Function CalculateDistance(ByVal signalLevelInDb As Double, _
                                      Optional ByVal freqInMHz As Double = 2400) As Double
        Dim exp As Double = (27.55 - (20 * Math.Log10(freqInMHz)) + Math.Abs(signalLevelInDb)) / 20.0
        Return Math.Pow(10.0, exp)
    End Function

    ' ==========================================================================
    '  GetVendorFromFileOui
    ' ==========================================================================
    Public Function GetVendorFromFileOui(ByVal mac As String) As String
        Dim fname As String
        fname = Application.StartupPath + "\VendorsFiles\" + "oui.txt"
        If My.Computer.FileSystem.FileExists(fname) Then
            mac = Left(mac, 8)
            Dim sa As String() = IO.File.ReadAllLines(fname)
            For i As Int32 = 0 To sa.Length - 1
                If sa(i).StartsWith(mac) Then
                    Dim s As String = Mid(sa(i), 17)
                    s = s.Replace(vbTab, " ").Trim
                    sa = s.Split(" "c)
                    s = sa(0)
                    If sa.Length > 1 Then s += " " + sa(1)
                    Return Left(s, 20)
                End If
            Next
        End If
        Return ""
    End Function

    ' ==========================================================================
    '  FindMaxRate
    ' ==========================================================================
    Friend Function FindMaxRate(ByVal RateSet() As UShort) As String
        Dim max As Int32 = -1
        For i As Int32 = 0 To RateSet.Length - 1
            Dim rate As Int32 = (RateSet(i) And &H7FFF) \ 2
            If rate > max Then max = rate
        Next
        Return max.ToString + " Mbps"
    End Function

    Friend Function FindMaxRate(ByVal RateSet As Generic.List(Of Double)) As Int32
        Dim max As Int32 = -1
        For i As Int32 = 0 To RateSet.Count - 1
            Dim rate As Int32 = CInt(RateSet(i))
            If rate > max Then max = rate
        Next
        Return max
    End Function


    ' ==========================================================================
    '  Compare Strings
    ' ==========================================================================
    Public Function CompareStrings_AlsoNumerics(ByVal s1 As String, ByVal s2 As String) As Int32
        ' -------------------------------------------------------- 
        If String.IsNullOrEmpty(s1) AndAlso String.IsNullOrEmpty(s2) Then
            Return 0
        End If
        If String.IsNullOrEmpty(s1) Then
            Return -1
        End If
        If String.IsNullOrEmpty(s2) Then
            Return 1
        End If
        ' --------------------------------------------------------
        s1 = s1.ToUpper
        s2 = s2.ToUpper
        ' -------------------------------------------------------- 
        Dim c1 As Char = s1(0)
        Dim c2 As Char = s2(0)
        Dim b1 As Boolean = Char.IsDigit(c1)
        Dim b2 As Boolean = Char.IsDigit(c2)
        ' -------------------------------------------------------- One is a digit, the other isn't : Fast exit
        If b1 <> b2 Then
            Return s1.CompareTo(s2)
        End If
        ' -------------------------------------------------------- 
        Dim len1 As Int32 = s1.Length
        Dim len2 As Int32 = s2.Length
        Dim pos2 As Int32 = 0
        Dim pos1 As Int32 = 0
        Dim bIsDigit As Boolean = b1
        ' -------------------------------------------------------- 
        While True
            ' ----------------------------------------------------
            If bIsDigit Then
                ' ------------------------------------------------ Do number comparison
                Dim n1 As Long = 0
                Dim n2 As Long = 0
                ' ------------------------------------------------ Convert the sequence of digits to a long number
                Do
                    c1 = s1(pos1)
                    b1 = Char.IsDigit(c1)
                    If b1 Then
                        n1 = n1 * 10 + Asc(c1) - Asc("0"c)
                    End If
                    ' -------------------------------------------- Stop if non-digit or reached the end of one string
                    pos1 += 1
                Loop While b1 AndAlso pos1 < len1
                ' ------------------------------------------------ Convert the sequence of digits to a long number
                Do
                    c2 = s2(pos2)
                    b2 = Char.IsDigit(c2)
                    If b2 Then
                        n2 = n2 * 10 + Asc(c2) - Asc("0"c)
                    End If
                    ' -------------------------------------------- Stop if non-digit or reached the end of one string
                    pos2 += 1
                Loop While b2 AndAlso pos2 < len2
                ' ------------------------------------------------ Compare the numbers
                If (n1 < n2) Then
                    Return -1
                End If
                If (n1 > n2) Then
                    Return 1
                End If
                ' Here, the numbers are equal. If we reached the end of the strings,
                ' we say they are equal, otherwise we continue on comparing strings
                If pos1 = len1 AndAlso pos2 = len2 Then
                    Return 0
                End If

            Else
                ' ------------------------------------------------- Do string comparison, character by character
                Do
                    c1 = s1(pos1)
                    c2 = s2(pos2)
                    b1 = Not Char.IsDigit(c1)
                    b2 = Not Char.IsDigit(c2)
                    ' --------------------------------------------- Two non-digits, different
                    If b1 AndAlso b2 AndAlso c1 <> c2 Then
                        Return Asc(c1) - Asc(c2) ' Just compare these different chars
                    End If
                    ' --------------------------------------------- One is digit, and the other isn't one
                    If (b1 AndAlso Not b2) OrElse (Not b1 AndAlso b2) Then
                        Return Asc(c1) - Asc(c2) ' Just compare these different chars
                    End If
                    ' --------------------------------------------- Next chars
                    pos1 += 1
                    pos2 += 1
                    ' --------------------------------------------- Stop if digit or reached the end of one string
                Loop While b1 AndAlso b2 AndAlso pos1 < len1 AndAlso pos2 < len2
                ' -------------------------------------------------
                If b1 AndAlso pos1 = len1 AndAlso pos2 = len2 Then
                    Return 0 ' end with non-digits without finding differences
                End If
            End If
            ' ----------------------------------------------------- Have we reached one end?
            If pos1 = len1 AndAlso len1 < len2 Then
                Return -1 ' --------------------------------------- s1 is shorter, so smaller (all chars were equal so far)
            End If
            If pos2 = len2 AndAlso len2 < len1 Then
                Return 1 ' ---------------------------------------- s2 is shorter, so smaller (all chars were equal so far)
            End If
            ' -----------------------------------------------------
            ' Not at the end, we stopped on different kind of char (digit vs. non-digits), let's process them
            ' -----------------------------------------------------
            ' ----------------------------------------------------- Compared chars, we went one char too far, into digits
            If Not bIsDigit Then
                ' ------------------------------------------------- Put back current chars into the comparison
                pos1 -= 1
                pos2 -= 1
            End If
            ' ----------------------------------------------------- Switch the comparison mode
            bIsDigit = Not bIsDigit
        End While
    End Function


    ' ==========================================================================
    '  LIST NETWORKS WITH NETSH
    ' ==========================================================================
    Public Function ListNetworksWithNetsh() As String
        Dim info As ProcessStartInfo = New ProcessStartInfo("netsh", "wlan show interfaces")
        'Dim info As ProcessStartInfo = New ProcessStartInfo("netsh", "wlan show profiles")
        'Dim info As ProcessStartInfo = New ProcessStartInfo("netsh", "wlan show profile NETGEAR82")
        'Dim info As ProcessStartInfo = New ProcessStartInfo("netsh", "wlan show drivers")
        'info.WorkingDirectory = "@%WINDIR%\system32"
        info.WindowStyle = System.Diagnostics.ProcessWindowStyle.Hidden
        info.CreateNoWindow = True
        info.RedirectStandardOutput = True
        info.UseShellExecute = False
        Dim proc As System.Diagnostics.Process = New System.Diagnostics.Process
        proc.StartInfo = info
        proc.Start()
        Return proc.StandardOutput.ReadToEnd()
    End Function

    ' --------------------
    '  OUTPUT EXAMPLE
    ' --------------------
    '"\r\n There is 1 interface on the system: \r\n\r\n
    'Name                   : Wireless Network Connection\r\n
    'Description            : Atheros AR9285 Wireless Network Adapter\r\n
    'GUID                   : xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx\r\n
    'Physical address       : xx:xx:xx:xx:xx:xx\r\n
    'State                  : connected\r\n
    'SSID                   : Dynex2\r\n
    'BSSID                  : xx:xx:xx:xx:xx:xx\r\n
    'Network type           : Infrastructure\r\n
    'Radio type             : 802.11g\r\n
    'Authentication         : WPA2-Personal\r\n
    'Cipher                 : CCMP\r\n
    'Connection mode        : Auto Connect\r\n
    'Channel                : 1\r\n
    'Receive rate (Mbps)    : 54\r\n
    'Transmit rate (Mbps)   : 54\r\n
    'Signal                 : 100% \r\n
    'Profile                : Dynex2 \r\n\r\n
    'Hosted network status  : Not available\r\n\r\n"

End Module
