
Imports System.Runtime.InteropServices
Imports System.Text

Public Class WlanClient
#Region "Fields"

    Private ReadOnly _clientHandle As IntPtr
    Private ReadOnly _ifaces As New Dictionary(Of Guid, WlanInterface)()
    Private _negotiatedVersion As UInteger

#End Region

#Region "Properties"

    Public ReadOnly Property Interfaces() As WlanInterface()
        Get
            Dim ptr As IntPtr
            Dim interfaceArray2 As WlanInterface()
            Dim ret As Int32 = Wlan.WlanEnumInterfaces(_clientHandle, IntPtr.Zero, ptr)
            If ret <> 0 Then Return Nothing

            Try
                Dim [structure] As Wlan.WlanInterfaceInfoListHeader = CType(Marshal.PtrToStructure(ptr, GetType(Wlan.WlanInterfaceInfoListHeader)), Wlan.WlanInterfaceInfoListHeader)
                Dim num As Long = ptr.ToInt64() + Marshal.SizeOf([structure])
                Dim interfaceArray As WlanInterface() = New WlanInterface(CInt([structure].numberOfItems - 1)) {}
                Dim list As New List(Of Guid)()
                For i As Integer = 0 To CInt([structure].numberOfItems - 1)
                    Dim info As Wlan.WlanInterfaceInfo = CType(Marshal.PtrToStructure(New IntPtr(num), GetType(Wlan.WlanInterfaceInfo)), Wlan.WlanInterfaceInfo)
                    num += Marshal.SizeOf(info)
                    list.Add(info.interfaceGuid)
                    Dim interface2 As WlanInterface = If(_ifaces.ContainsKey(info.interfaceGuid), _ifaces(info.interfaceGuid), New WlanInterface(Me, info))
                    interfaceArray(i) = interface2
                    _ifaces(info.interfaceGuid) = interface2
                Next
                Dim queue As New Queue(Of Guid)()
                For Each guid As Guid In _ifaces.Keys
                    If Not list.Contains(guid) Then
                        queue.Enqueue(guid)
                    End If
                Next
                While queue.Count <> 0
                    Dim key As Guid = queue.Dequeue()
                    _ifaces.Remove(key)
                End While
                interfaceArray2 = interfaceArray
            Finally
                Wlan.WlanFreeMemory(ptr)
            End Try
            Return interfaceArray2
        End Get
    End Property

    Public ReadOnly Property ItsClientHandle() As IntPtr
        Get
            Return _clientHandle
        End Get
    End Property

    'private Logger ItsLogger
    '{
    '    get;
    '    set;
    '}

    Private Property WlanNotificationCallback() As Wlan.WlanNotificationCallbackDelegate
        Get
            Return m_WlanNotificationCallback
        End Get
        Set(ByVal value As Wlan.WlanNotificationCallbackDelegate)
            m_WlanNotificationCallback = Value
        End Set
    End Property
    Private m_WlanNotificationCallback As Wlan.WlanNotificationCallbackDelegate

#End Region

#Region "Event Fields"

    'public RegisteredEventHandler<InterfaceNotificationEventsArgs> InterfaceArrivedEvent = new RegisteredEventHandler<InterfaceNotificationEventsArgs>();
    'public RegisteredEventHandler<InterfaceNotificationEventsArgs> InterfaceRemovedEvent = new RegisteredEventHandler<InterfaceNotificationEventsArgs>();

#End Region

#Region "Constructors"

    Public Sub New()
        Try
            Wlan.WlanOpenHandle(1, IntPtr.Zero, _negotiatedVersion, _clientHandle)
            WlanNotificationCallback = New Wlan.WlanNotificationCallbackDelegate(AddressOf OnWlanNotification)

            Dim source As Wlan.WlanNotificationSource
            Wlan.WlanRegisterNotification(_clientHandle, Wlan.WlanNotificationSource.All, False, WlanNotificationCallback, IntPtr.Zero, IntPtr.Zero, source)
        Catch
            Wlan.WlanCloseHandle(_clientHandle, IntPtr.Zero)
        End Try
    End Sub

    Protected Overrides Sub Finalize()
        Try
            Wlan.WlanCloseHandle(_clientHandle, IntPtr.Zero)
        Finally
            MyBase.Finalize()
        End Try
    End Sub

#End Region

#Region "Public Methods"

    Public Function GetStringForReasonCode(ByVal reasonCode As Wlan.WlanReasonCode) As String
        Dim stringBuffer As New StringBuilder(&H400)
        Wlan.WlanReasonCodeToString(reasonCode, stringBuffer.Capacity, stringBuffer, IntPtr.Zero)
        Return stringBuffer.ToString()
    End Function

#End Region

#Region "Private Methods"

    Private Sub OnWlanNotification(ByRef notifyData As Wlan.WlanNotificationData, ByVal context As IntPtr)
        Dim interface2 As WlanInterface = If(_ifaces.ContainsKey(notifyData.interfaceGuid), _ifaces(notifyData.interfaceGuid), Nothing)
        Select Case notifyData.notificationSource
            Case Wlan.WlanNotificationSource.Acm
                Select Case CInt(notifyData.NotificationCode)
                    Case 8
                        If notifyData.dataSize >= Marshal.SizeOf(0) Then
                            Dim reasonCode As Wlan.WlanReasonCode = CType(Marshal.ReadInt32(notifyData.dataPtr), Wlan.WlanReasonCode)
                            If interface2 IsNot Nothing Then
                                interface2.OnWlanReason(notifyData, reasonCode)
                            End If
                        End If
                        GoTo Label_0194

                    Case 9, 10, 11, 20, &H15
                        If True Then
                            Dim nullable As System.Nullable(Of Wlan.WlanConnectionNotificationData) = ParseWlanConnectionNotification(notifyData)
                            If nullable.HasValue AndAlso (interface2 IsNot Nothing) Then
                                interface2.OnWlanConnection(notifyData, nullable.Value)
                            End If
                            GoTo Label_0194
                        End If
                    Case 12, 15, &H10, &H11, &H12, &H13
                        GoTo Label_0194

                    Case 13
                        'InterfaceArrivedEvent.Raise(this, new InterfaceNotificationEventsArgs(notifyData.interfaceGuid));
                        GoTo Label_0194

                    Case 14
                        'InterfaceRemovedEvent.Raise(this, new InterfaceNotificationEventsArgs(notifyData.interfaceGuid));
                        GoTo Label_0194
                End Select
                Exit Select

            Case Wlan.WlanNotificationSource.Msm
                Select Case CInt(notifyData.NotificationCode)
                    Case 1, 2, 3, 4, 5, 6, _
                            9, 10, 11, 12, 13
                        If True Then
                            Dim nullable2 As System.Nullable(Of Wlan.WlanConnectionNotificationData) = ParseWlanConnectionNotification(notifyData)
                            If nullable2.HasValue AndAlso (interface2 IsNot Nothing) Then
                                interface2.OnWlanConnection(notifyData, nullable2.Value)
                            End If
                            GoTo Label_0194
                        End If
                    Case 7, 8
                        GoTo Label_0194
                End Select
                GoTo Label_0194
        End Select
Label_0194:
        If interface2 IsNot Nothing Then
            interface2.OnWlanNotification(notifyData)
        End If
    End Sub

    Private Function ParseWlanConnectionNotification(ByRef notifyData As Wlan.WlanNotificationData) As System.Nullable(Of Wlan.WlanConnectionNotificationData)
        Dim num As Integer = Marshal.SizeOf(GetType(Wlan.WlanConnectionNotificationData))
        If notifyData.dataSize < num Then
            Return Nothing
        End If
        Dim data As Wlan.WlanConnectionNotificationData = CType(Marshal.PtrToStructure(notifyData.dataPtr, GetType(Wlan.WlanConnectionNotificationData)), Wlan.WlanConnectionNotificationData)
        If data.wlanReasonCode = Wlan.WlanReasonCode.Success Then
            Dim ptr As New IntPtr(notifyData.dataPtr.ToInt64() + Marshal.OffsetOf(GetType(Wlan.WlanConnectionNotificationData), "profileXml").ToInt64())
            data.profileXml = Marshal.PtrToStringUni(ptr)
        End If
        Return data
    End Function

#End Region
End Class

